
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include "app_uart.h"
#include "app_error.h"
#include "nrf_delay.h"
#include "nrf.h"
#include "bsp.h"
#if defined (UART_PRESENT)
#include "nrf_uart.h"
#endif
#if defined (UARTE_PRESENT)
#include "nrf_uarte.h"
#endif

#define MAX_TEST_DATA_BYTES     (15U)                /**< max number of test bytes to be used for tx and rx. */
#define UART_TX_BUF_SIZE 256                         /**< UART TX buffer size. */
#define UART_RX_BUF_SIZE 1                         /**< UART RX buffer size. */

void uart_error_handle(app_uart_evt_t * p_event)
{
    switch (p_event->evt_type)
    {
        case APP_UART_COMMUNICATION_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_communication);
            break;

        case APP_UART_FIFO_ERROR:
            APP_ERROR_HANDLER(p_event->data.error_code);
            break;

        case APP_UART_DATA_READY:
            printf("APP_UART_DATA_READY");
            break;

        case APP_UART_TX_EMPTY:
            printf("APP_UART_TX_EMPTY");
            printf("An event indicating that UART has completed transmission of all available data in the TX FIFO.");
            app_uart_flush();
            break;

        case APP_UART_DATA:
            printf("APP_UART_DATA");
            printf("An event indicating that UART data has been received, and data is present in data field. This event is only used when no FIFO is configured.");
            break;

        default:
            break;
    }
}

/* When UART is used for communication with the host do not use flow control.*/
#define UART_HWFC APP_UART_FLOW_CONTROL_DISABLED

/**
 * @brief Function for main application entry.
 */
int main(void)
{
    uint32_t err_code;

    uint32_t i = 0;

    bsp_board_init(BSP_INIT_LEDS);

    const app_uart_comm_params_t comm_params =
      {
          14, /* UART RX <-> LN610 TX */
          13, /* UART TX <-> LN610 RX */
          UART_PIN_DISCONNECTED,
          UART_PIN_DISCONNECTED,
          UART_HWFC,
          false,
#if defined (UART_PRESENT)
          NRF_UART_BAUDRATE_9600
#else
          NRF_UARTE_BAUDRATE_9600
#endif
      };

    APP_UART_FIFO_INIT(&comm_params,
                         UART_RX_BUF_SIZE,
                         UART_TX_BUF_SIZE,
                         uart_error_handle,
                         APP_IRQ_PRIORITY_LOWEST,
                         err_code);

    APP_ERROR_CHECK(err_code);

/* Test */
    uint8_t tx_data[4] = { 0x24, 0x01, 0x00, 0x25 };
    uint8_t rx_data = 0;
    uint8_t buff[255];

    // Start sending one byte
    for (i = 0; i < 4; i++)
    {
        app_uart_put(tx_data[i]);
    }

    // Receive data
    i = 0;
    while (true)
    {
         app_uart_get(&rx_data);

         buff[i++] = rx_data;
         if (i % 255)
            break;
    }

    return 0;
}
